Public Module Game

    Public Version As String = "1.8"

    Public RND As New Random(Now.Millisecond)

    Public GameTime As DateTime = Date.Now

    Public DoNewGame As Boolean
    Public OddTurn As Boolean = False

    Public GameWin As Boolean
    Public GameLose As Boolean

    ' initial resource values
    Public Water As Integer = 16
    Public Supplies As Integer = 5
    Public Ammo As Integer = 3
    Public Medicine As Integer = 2

    Public NightAssaultSuccess As Boolean = True

    Public RaidEveryHour As Boolean = True

    ' counters for certain events. don't think TimeToRestock is used...
    Public TimeToDrink As Integer
    Public TimeToRestock As Integer

    Public OutOfWater As Boolean

    ' raid priorities, array is filled in Initialise()
    Public Priorities(4, 1) As Integer

    ' starting priorities
    Public RaidPriority_Survivors As Integer = 1
    Public RaidPriority_Weapons As Integer = 2
    Public RaidPriority_AmmoSupplies As Integer = 2
    Public RaidPriority_MedicineWater As Integer = 3

    ' initial hours a player must survive
    Public HoursLeft As Integer = 60

    ' base zombie stats
    Public ZombieHitPoints As Integer = 6 ' 6hp
    Public MeleeInfectionRisk As Integer = 25 ' 25% chance to infect on bite
    Public MeleeBiteChance As Integer = 60
    ' 60% chance to bite an unarmed survivor (increased from 50 to 60% as of v1.5)
    ' 50% chance to bite survivor with melee weapon
    ' 40% chance to bite a survivor with a melee weapon and Grizzled perk
    ' 30& chance to bite a survivor with a loaded ranged weapon
    ' 20% chance to bite a survivor with a loaded ranged weapon and Grizzled perk

    ' stats for endgame
    Public ZombiesKilled As Integer
    Public SurvivorsLost As Integer
    Public SurvivorsTurned As Integer
    Public SurvivorsInfected As Integer

    ' zombie mass index, used to calculate encounter sizes and chance
    Public ZombieMass As Integer
    Public ZombieMassMinimum As Integer = 16

    Public RandomFirstNames As New Generic.List(Of String)
    Public RandomLastNames As New Generic.List(Of String)
    Public CustomNames As New Generic.List(Of String)

    Public Survivors As New Generic.List(Of Person)
    Public Weapons As New Generic.List(Of WeaponType)

    Public SelectedEncounter As New EncounterItem
    Public SurvivorToPromote As New Person

    Public Buildings As Integer = 5
    Public BuildingNames(5) As String

    Public BuildingBonus_Ammo As Integer = 5
    Public BuildingBonus_Supplies As Integer = 5
    Public BuildingBonus_Medicine As Integer = 2
    Public BuildingBonus_Water As Integer = 4

    Public RaidBonus_Ammo As Integer = 4
    Public RaidBonus_Supplies As Integer = 8
    Public RaidBonus_Medicine As Integer = 1
    Public RaidBonus_Water As Integer = 4

    ' patch 1.5 - special bonuses for doing certain things
    Public SpecialBonus_AllBuildingsHeld As Boolean

    ' if you want to chance the number of hours it takes for a player to
    ' get a promotion, here is where you need to be
    ' doesn't chance the promote window text - or maybe it does!
    Public Enum SurvivorLevels
        Level1 = 8
        Level2 = 16
        Level3 = 24
    End Enum

    ' if you want to add a new event, start here
    Public Enum EventTypes
        Secure_Building
        Attacked_Building
        Raid
        Alert
        Attacked_Outside
        ' SomeNewEvent1
        ' SomeNewEvent2
        ' etc.
    End Enum

    Public Class EncounterItem
        Public Description As String
        Public ResolvedDescription As String
        Public Failed As Boolean
        Public Played As Boolean

        Public Building As BuildingInfo
        Public DeadPeople As New Generic.List(Of Person)

        Public EventType As EventTypes

        Public Overrides Function tostring() As String
            Return Description
        End Function

        Public ReadOnly Property GetDescription()
            Get
                Return Description
            End Get
        End Property
    End Class

    Public Class BuildingInfo
        Public Name As String
        Public People As Generic.List(Of Person)
        Public AttackChance As Integer
        Public LastAttack As Integer
        Public Reinforcement As Integer
        Public Size As Integer
        Public Secured As Boolean
        ' not used - buildings just become unsecured instead
        Public Overrun As Boolean
    End Class

    Public Class Person
        Public Name As String
        Public Wounds As Integer
        Public Experience As Integer
        Public Courage As Integer

        Public Level As Integer

        Public Dead As Boolean

        Public Infected As Boolean
        ' initially had food, changed to water but the var stayed as 'Starved'
        Public Starved As Boolean

        Public Tired As Boolean ' not used
        Public Scared As Boolean ' not used

        Public Weapon As WeaponType
        Public Ammo As Integer
        Public Grenades As Integer ' not used

        Public Promo_Grizzled As Boolean
        Public Promo_SharpEye As Boolean
        Public Promo_Conserve As Boolean
        Public Promo_SteadyHand As Boolean
        Public Promo_Lucky As Boolean
        Public Promo_Home As Boolean

        Public Overrides Function ToString() As String
            Return GetListState
        End Function

        Public ReadOnly Property GetName() As String
            Get
                Return Name
            End Get
        End Property

        Public ReadOnly Property GetListState() As String
            Get
                Dim stateString As String = ""
                stateString &= Name & " (H: " & Experience & ") - " & Weapon.Name & " (A: " & Ammo & ")"
                If Wounds > 0 Then
                    stateString = "+ " & stateString
                ElseIf Starved = True Or Infected = True Then
                    stateString = "! " & stateString
                End If

                Dim Promos As String = " ["

                If Promo_Grizzled Then Promos &= "G"
                If Promo_SharpEye Then Promos &= "E"
                If Promo_Conserve Then Promos &= "|C"
                If Promo_SteadyHand Then Promos &= "|S"
                If Promo_Lucky Then Promos &= "|L"
                If Promo_Home Then Promos &= "|H"

                If Promos = " [" Then
                    Promos = ""
                Else
                    Promos &= "]"
                End If

                stateString &= Promos

                Return stateString
            End Get
        End Property

    End Class

    Public Class WeaponType
        Public Name As String
        Public Damage As Integer

        Public IsMelee As Boolean
        Public AmmoNeed As Integer

        Public Overrides Function ToString() As String
            Return GetName
        End Function

        Public ReadOnly Property GetName() As String
            Get
                Return Name
            End Get
        End Property
    End Class

    Private Function GenerateRandomName()
        Dim First As String, Last As String
        Dim FinalName As String

        If CustomNames.Count = 0 Then
            First = RandomFirstNames(RND.Next(RandomFirstNames.Count - 1))
            Last = RandomLastNames(RND.Next(RandomLastNames.Count - 1))
            FinalName = First & " " & Last
        Else
            Dim rndName As Integer = RND.Next(0, CustomNames.Count)
            FinalName = CustomNames(rndName)
            CustomNames.RemoveAt(rndName)
        End If

        Return FinalName
    End Function

    Public Function GenerateNewPerson() As Person
        Dim newPerson As New Person

        newPerson.Name = GenerateRandomName()
        newPerson.Weapon = Fists()

        Return newPerson
    End Function

    Public Function IsNight() As Boolean
        If (GameTime.ToString("tt") = "PM" And GameTime.Hour > 18) Or (GameTime.ToString("tt") = "AM" And GameTime.Hour < 6) Then
            Return True
        End If

        Return False
    End Function

    ' next two functions tell us if it's the first hour of darkness or light
    Public Function IsStartNight() As Boolean
        If GameTime.ToString("tt") = "PM" And GameTime.Hour = 19 Then
            Return True
        End If

        Return False
    End Function

    Public Function IsStartDay() As Boolean
        If GameTime.ToString("tt") = "AM" And GameTime.Hour = 6 Then
            Return True
        End If

        Return False
    End Function

    ' weapon definitions. if you want to add new weapons,
    ' start here. keep in mind if you want them to appear
    ' in the game, you will need to seed them in the raid code
    ' in frmEncounter
    Public Function Bat() As WeaponType
        Dim tempWeapon As New WeaponType

        With tempWeapon
            .Name = "Baseball bat"
            .Damage = 2
            .AmmoNeed = 0
            .IsMelee = True
        End With

        Return tempWeapon
    End Function

    Public Function Fists() As WeaponType
        Dim tempWeapon As New WeaponType

        With tempWeapon
            .Name = "Fists"
            .Damage = 1
            .AmmoNeed = 0
            .IsMelee = True
        End With

        Return tempWeapon
    End Function

    Public Function Machete() As WeaponType
        Dim tempWeapon As New WeaponType

        With tempWeapon
            .Name = "Machete"
            .Damage = 3
            .AmmoNeed = 0
            .IsMelee = True
        End With

        Return tempWeapon
    End Function

    Public Function Pistol() As WeaponType
        Dim tempWeapon As New WeaponType

        With tempWeapon
            .Name = "Pistol"
            .Damage = 2
            .AmmoNeed = 1
            .IsMelee = False
        End With

        Return tempWeapon
    End Function

    Public Function Shotgun() As WeaponType
        Dim tempWeapon As New WeaponType

        With tempWeapon
            .Name = "Shotgun"
            .Damage = 6
            .AmmoNeed = 3
            .IsMelee = False
        End With

        Return tempWeapon
    End Function

    Public Function Rifle() As WeaponType
        Dim tempWeapon As New WeaponType

        With tempWeapon
            .Name = "Rifle"
            .Damage = 3
            .AmmoNeed = 2
            .IsMelee = False
        End With

        Return tempWeapon
    End Function

    ' boots our random name generator and raid chance ranges
    Public Sub Initialise()

        Dim CustomNameFile As String = My.Application.Info.DirectoryPath & "\names.txt"

        If My.Computer.FileSystem.FileExists(CustomNameFile) Then
            Dim nameReader As New System.IO.StreamReader(CustomNameFile)

            Dim sLine As String = ""
            Do While nameReader.EndOfStream = False
                sLine = nameReader.ReadLine
                If sLine.Substring(0, 2) <> "//" Then
                    CustomNames.Add(sLine)
                End If
            Loop

            nameReader.Close()
        End If

        RandomFirstNames.Add("Bob")
        RandomFirstNames.Add("Michael")
        RandomFirstNames.Add("John")
        RandomFirstNames.Add("Simon")
        RandomFirstNames.Add("Anthony")
        RandomFirstNames.Add("James")
        RandomFirstNames.Add("Samuel")
        RandomFirstNames.Add("Tim")
        RandomFirstNames.Add("David")
        RandomFirstNames.Add("Allen")
        RandomFirstNames.Add("Jake")
        RandomFirstNames.Add("Richard")

        RandomFirstNames.Add("Lisa")
        RandomFirstNames.Add("Sarah")
        RandomFirstNames.Add("Olivia")
        RandomFirstNames.Add("Rachael")
        RandomFirstNames.Add("Christine")
        RandomFirstNames.Add("Regina")
        RandomFirstNames.Add("Hayley")
        RandomFirstNames.Add("Kim")
        RandomFirstNames.Add("Melanie")

        RandomFirstNames.Add("Trevor")
        RandomFirstNames.Add("Davis")
        RandomFirstNames.Add("Gregory")
        RandomFirstNames.Add("Julian")
        RandomFirstNames.Add("Bob")
        RandomFirstNames.Add("Brian")
        RandomFirstNames.Add("Fred")
        RandomFirstNames.Add("Chris")
        RandomFirstNames.Add("Cameron")
        RandomFirstNames.Add("Oscar")

        RandomFirstNames.Add("Amanda")
        RandomFirstNames.Add("Anne")
        RandomFirstNames.Add("Rebecca")
        RandomFirstNames.Add("Wendy")
        RandomFirstNames.Add("Tanya")
        RandomFirstNames.Add("Belinda")
        RandomFirstNames.Add("Nancy")
        RandomFirstNames.Add("Patricia")
        RandomFirstNames.Add("Sally")
        RandomFirstNames.Add("Julia")

        RandomFirstNames.Add("Oliver")
        RandomFirstNames.Add("Brad")
        RandomFirstNames.Add("Harris")
        RandomFirstNames.Add("Ivan")
        RandomFirstNames.Add("Boris")
        RandomFirstNames.Add("Graham")
        RandomFirstNames.Add("Edward")
        RandomFirstNames.Add("Zack")
        RandomFirstNames.Add("George")

        RandomFirstNames.Add("Gloria")
        RandomFirstNames.Add("Vanessa")
        RandomFirstNames.Add("Danielle")
        RandomFirstNames.Add("Fran")
        RandomFirstNames.Add("Cindy")
        RandomFirstNames.Add("Samatha")
        RandomFirstNames.Add("Cassie")
        RandomFirstNames.Add("Jane")
        RandomFirstNames.Add("Xena")

        RandomFirstNames.Add("Craig")
        RandomFirstNames.Add("Ben")
        RandomFirstNames.Add("Jacob")
        RandomFirstNames.Add("Nathan")
        RandomFirstNames.Add("Nick")
        RandomFirstNames.Add("Francis")
        RandomFirstNames.Add("Peter")
        RandomFirstNames.Add("Paul")
        RandomFirstNames.Add("Frank")
        RandomFirstNames.Add("Victor")
        RandomFirstNames.Add("William")
        RandomFirstNames.Add("Brock")
        RandomFirstNames.Add("Liam")
        RandomFirstNames.Add("Henry")
        RandomFirstNames.Add("Matthew")

        RandomFirstNames.Add("Peta")
        RandomFirstNames.Add("Lana")
        RandomFirstNames.Add("Holly")
        RandomFirstNames.Add("Nicole")
        RandomFirstNames.Add("Stephanie")
        RandomFirstNames.Add("Brooke")
        RandomFirstNames.Add("Louise")
        RandomFirstNames.Add("Linda")
        RandomFirstNames.Add("Natalie")
        RandomFirstNames.Add("Sharon")
        RandomFirstNames.Add("Crystal")
        RandomFirstNames.Add("April")
        RandomFirstNames.Add("Summer")
        RandomFirstNames.Add("Autumn")
        RandomFirstNames.Add("Kathleen")
        RandomFirstNames.Add("Tamara")
        RandomFirstNames.Add("Jessica")
        RandomFirstNames.Add("Jade")

        RandomLastNames.Add("Wilson")
        RandomLastNames.Add("Cross")
        RandomLastNames.Add("Smith")
        RandomLastNames.Add("Locke")
        RandomLastNames.Add("Wong")
        RandomLastNames.Add("Poe")
        RandomLastNames.Add("Lovecraft")
        RandomLastNames.Add("Hemmingway")
        RandomLastNames.Add("Freud")
        RandomLastNames.Add("Hubbard")
        RandomLastNames.Add("Coontz")
        RandomLastNames.Add("Howard")
        RandomLastNames.Add("Weinstein")
        RandomLastNames.Add("Spielberg")
        RandomLastNames.Add("Lucas")
        RandomLastNames.Add("Craven")
        RandomLastNames.Add("Grimm")
        RandomLastNames.Add("Cohen")
        RandomLastNames.Add("Wright")
        RandomLastNames.Add("Lewis")
        RandomLastNames.Add("Tolkien")
        RandomLastNames.Add("Romero")
        RandomLastNames.Add("Carroll")
        RandomLastNames.Add("Shakespeare")
        RandomLastNames.Add("Bay")
        RandomLastNames.Add("Fincher")
        RandomLastNames.Add("Ritchie")
        RandomLastNames.Add("Fitzgerald")
        RandomLastNames.Add("Carey")
        RandomLastNames.Add("Lee")
        RandomLastNames.Add("Jackson")
        RandomLastNames.Add("Ford")
        RandomLastNames.Add("Schwartz")
        RandomLastNames.Add("Robinson")
        RandomLastNames.Add("Jones")
        RandomLastNames.Add("Murphy")
        RandomLastNames.Add("Walker")
        RandomLastNames.Add("Avery")
        RandomLastNames.Add("Law")
        RandomLastNames.Add("Newton")
        RandomLastNames.Add("Gibson")
        RandomLastNames.Add("McCain")
        RandomLastNames.Add("Federer")
        RandomLastNames.Add("Woods")
        RandomLastNames.Add("Taylor")
        RandomLastNames.Add("Overton")
        RandomLastNames.Add("Hewitt")

        BuildingNames(1) = "Store"
        BuildingNames(2) = "Hospital"
        BuildingNames(3) = "Warehouse"
        BuildingNames(4) = "Factory"
        BuildingNames(5) = "Mansion"

        ' raid chance values - 1-70%, 71-90% and 91-100%
        Priorities(1, 0) = 1
        Priorities(1, 1) = 70

        Priorities(2, 0) = 71
        Priorities(2, 1) = 85

        Priorities(3, 0) = 86
        Priorities(3, 1) = 95

        Priorities(4, 0) = 96
        Priorities(4, 1) = 100

        ' game start date - Aus release of 28 Days Later (best zombie flick ever made)
        GameTime = Date.Parse("September 4, 2003 6am")
    End Sub

    ' calculates the chances of meeting a zombie. fiddle with this to increase/decrease chances
    Public Function ZombieChance() As Integer
        Dim Chance As Integer = 8

        Chance += (ZombieMass / 3)

        If Chance > 100 Then Chance = 100

        If FinalHour() Then Chance = 100

        Return Chance
    End Function

    ' calculates the number of zombies found per encounter
    Public Function ZombieParty() As Integer
        Dim Density As Integer = (ZombieMass / 3)

        If Density = 0 Then Density = 1

        Return RND.Next(Density, Density * 1.3)
    End Function

    Public Function FinalHour() As Boolean
        If HoursLeft = 1 Then Return True
    End Function

End Module
